<?php
/**
 * WC_Gateway_PayMe class
 *
 * @author   SomewhereWarm <info@somewherewarm.gr>
 * @package  WooCommerce PayMe Payments Gateway
 * @since    1.0.0
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * PayMe Gateway.
 *
 * @class    WC_Gateway_PayMe
 * @version  1.0.7
 */
class WC_Gateway_PayMe extends WC_Payment_Gateway {

	/**
	 * Payment gateway instructions.
	 * @var string
	 *
	 */
	protected $instructions;

	/**
	 * Unique id for the gateway.
	 * @var string
	 *
	 */
	public $id = 'payme';

	/**
	 * Constructor for the gateway.
	 */
	public function __construct() {
		
		$this->icon               = 'https://firebasestorage.googleapis.com/v0/b/tygapay-ba6b4.appspot.com/o/images%2Ftygapay%2Ftyga_icon.png?alt=media&token=ea48d24a-2cfd-43f4-a11a-d74d48902baf';
		$this->has_fields         = true;
		$this->supports           = array(
			'products'
		);

		$this->method_title       = _x( 'TygaPay', 'PayMe payment method', 'woocommerce-gateway-payme' );
		$this->method_description = __( 'Pay using your crytocurrency.', 'woocommerce-gateway-payme' );

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		// Define user set variables.
		$this->title                    = $this->get_option( 'title' );
		$this->description              = $this->get_option( 'description' );
		$this->instructions             = $this->get_option( 'instructions', $this->description );

		// Actions.
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
    // You can also register a webhook here
    add_action('woocommerce_api_tygapay', array($this, 'webhook'));
	}

	/**
	 * Initialise Gateway Settings Form Fields.
	 */
	public function init_form_fields() {

		$this->form_fields = array(
			'enabled' => array(
				'title'   => __( 'Enable/Disable', 'woocommerce-gateway-payme' ),
				'type'    => 'checkbox',
				'label'   => __( 'Enable TygaPay', 'woocommerce-gateway-payme' ),
				'default' => 'yes',
			),
			'title' => array(
				'title'       => __( 'Title', 'woocommerce-gateway-payme' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce-gateway-payme' ),
				'default'     => _x( 'TygaPay', 'TygaPay method', 'woocommerce-gateway-payme' ),
				'desc_tip'    => true,
			),
			'description' => array(
				'title'       => __( 'Description', 'woocommerce-gateway-payme' ),
				'type'        => 'textarea',
				'description' => __( 'Payment method description that the customer will see on your checkout.', 'woocommerce-gateway-payme' ),
				'default'     => __( 'Pay using your crytocurrency.', 'woocommerce-gateway-payme' ),
				'desc_tip'    => true,
			),
			'tyga_order_description' => array(
				'title'       => __( 'Order Description', 'woocommerce-gateway-payme' ),
				'type'        => 'text',
				'description' => __( 'The description of the order that will be sent to TygaPay (Optional).', 'woocommerce-gateway-payme' ),
				'desc_tip'    => true,
			),
      'tyga_api_key' => array(
				'title'       => __( 'Live Api Key', 'woocommerce-gateway-payme' ),
				'type'        => 'text',
				'description' => __( 'Key supplied by Tyga Team.', 'woocommerce-gateway-payme' ),
				'desc_tip'    => true,
      ),
      'tyga_secret' => array(
				'title'       => __( 'Live Api Secret', 'woocommerce-gateway-payme' ),
				'type'        => 'password',
				'description' => __( 'Secret supplied by Tyga Team.', 'woocommerce-gateway-payme' ),
				'desc_tip'    => true,
      ),
		);
	}

	/**
	 * Process the payment and return the result.
	 *
	 * @param  int  $order_id
	 * @return array
	 */
	public function process_payment($orderId) {
    wc_get_logger()->debug('[process_payment] order_id: '.$orderId);
    global $woocommerce;

    // we need the order details
    $order = wc_get_order($orderId);

    $returnUrl = $order->get_checkout_order_received_url();
    $cancelUrl = $order->get_cancel_order_url();
    $rejectedUrl = $order->get_cancel_order_url();

    wc_get_logger()->debug('[process_payment] returnUrl: '.$returnUrl);
    wc_get_logger()->debug('[process_payment] cancelUrl: '.$cancelUrl);

    if ($order->get_total() == 0) {
      $order->payment_complete();
      return;
    }

    $this->tyga_order_description = $this->get_option('tyga_order_description');
    wc_get_logger()->debug('[process_payment] tyga_order_description: '.$this->tyga_order_description);

    $body = [
      'orderNumber' => $orderId,
      'amount' => $order->get_total(),
      'name' => 'Order #' . $orderId,
      'email' => $order->billing_email,
      'description' => $this->tyga_order_description,
      'service' => 'payme',
      'source' => 'woocommerce',
      'notifyUrl' => get_site_url().'/wc-api/tygapay?orderId=' . $orderId,
      'returnUrl' => $returnUrl,
      'returnCancelledUrl' => $cancelUrl,
      'returnRejectedUrl' => $rejectedUrl,
      'customerInformation' => [
        'firstName' => $order->get_billing_first_name(),
        'lastName' => $order->get_billing_last_name(),
        'phone' => $order->get_billing_phone(),
        'address' => [
          'line1' => $order->get_billing_address_1(),
          'line2' => $order->get_billing_address_2(),
          'city' => $order->get_billing_city(),
          'state' => $order->get_billing_state(),
          'postalCode' => $order->get_billing_postcode(),
          'country' => $order->get_billing_country(),
        ]
      ],
    ];
    $body = wp_json_encode($body);

    $this->tyga_api_key = $this->get_option('tyga_api_key');
    $this->tyga_secret = $this->get_option('tyga_secret');

    $headers = [
      'x-api-key' => $this->tyga_api_key,
      'x-api-secret' => $this->tyga_secret,
      'Content-Type' => 'application/json'
    ];

    $args = array(
      'headers' => $headers,
      'body' => $body,
    );

    $response = wp_remote_post('https://orders-v1-api-rdqehkur6a-ey.a.run.app/orders', $args);

    if (!is_wp_error($response)) {
      $respBody = json_decode($response['body'], true);
      wc_get_logger()->debug('[process_payment] respBody: '.json_encode($respBody));

      $tygapayOrderId = $respBody['data']['orderId'];
      $status = $respBody['data']['status'];
      wc_get_logger()->debug('[process_payment] status: '.$status);

      // only process the order if it is in new status
      if ($status == "new") {
        $paymentUrl = $respBody['data']['paymentUrl'];
        wc_get_logger()->debug('[process_payment] paymentUrl: '.$paymentUrl);

        // add a note to the order
        $order->add_order_note('Your TygaPay Order Id is '.$tygapayOrderId, true);
        // update the order with the tygapayOrderId
        $order->update_meta_data('tygapayOrderId', $tygapayOrderId);
        $order->save();

        return array(
          'result' => 'success',
          'redirect' => $paymentUrl
        );
      } else {
        wc_add_notice('Order in '.$status.' status', 'error');
        $order->add_order_note('Order in '.$status.' status', true);
        $order->cancel_order();

        return array(
          'result' => 'success',
          'redirect' => $cancelUrl
        );
      }
    } else {
      wc_get_logger()->debug('[process_payment] responseError: '.$response->get_error_message());
      wc_add_notice('Connection error.', 'error');
      $order->add_order_note('Problem connecting to TygPay, check logs.', true);
    }
	}

  /*
  * Create a webhook to receive the payment status
  */
  public function webhook() {
    $orderId = $_GET['orderId'];
    wc_get_logger()->debug('[webhook] orderId: '.$orderId);

    global $woocommerce;

    // we received the payment
    $order = wc_get_order($orderId);
    $tygapayOrderId = $order->get_meta('tygapayOrderId');
    wc_get_logger()->debug('[webhook] order: '.$order);
    wc_get_logger()->debug('[webhook] tygapayOrderId: '.$tygapayOrderId);

    // if the tygapayOrderId is not set, then it's not a TygaPay order
    if (empty($tygapayOrderId)) {
      return;
    }

    $this->tyga_api_key = $this->get_option('tyga_api_key');
    $this->tyga_secret = $this->get_option('tyga_secret');

    $headers = [
      'x-api-key' => $this->tyga_api_key,
      'x-api-secret' => $this->tyga_secret,
      'Content-Type' => 'application/json'
    ];

    $args = array(
      'headers' => $headers,
    );

    $response = wp_remote_get('https://orders-v1-api-rdqehkur6a-ey.a.run.app/orders/'.$tygapayOrderId, $args);
    $respBody = json_decode($response['body'], true);

    $status = $respBody['data']['status'];
    wc_get_logger()->debug('[webhook] status: '.$status);

    if ($status == 'paid' || $status == 'success') {
      wc_get_logger()->debug('[webhook] order paid sucessfully');
      
      // Payment completed
      $order->payment_complete();
      wc_get_logger()->debug('[webhook] order marked as paid');

      // Reduce stock levels
      $order->reduce_order_stock();
      wc_get_logger()->debug('[webhook] reduce stock levels');

      // some notes to customer (replace true with false to make it private)
      $order->add_order_note('Your TygaPay order has been paid! Thank you!', true);
      wc_get_logger()->debug('[webhook] order note added - paid');

      // Empty cart
      $woocommerce->cart->empty_cart();
      wc_get_logger()->debug('[webhook] cart emptied');

    } elseif ($status == 'cancelled') {
        wc_get_logger()->debug('[webhook] order cancelled');

        // some notes to customer (replace true with false to make it private)
        $order->add_order_note('Your TygaPay order has been cancelled.', true);
        wc_get_logger()->debug('[webhook] order note added - cancelled');

        // Change the order status to cancelled
        $order->update_status('cancelled');
        wc_get_logger()->debug('[webhook] order status updated to cancelled');

    } elseif ($status == 'expired') {

        // some notes to customer (replace true with false to make it private)
        $order->add_order_note('Your TygaPay order has expired.', true);
        wc_get_logger()->debug('[webhook] order note added - expired');

        // Change the order status to expired
        $order->update_status('expired');
        wc_get_logger()->debug('[webhook] order status updated to expired');

    } else {
      wc_get_logger()->debug('[webhook] order status: '.$status);
      
      // Payment failed or expired
      $order->update_status($status);
      wc_get_logger()->debug('[webhook] order status updated to '.$status);

    }
    
  }
}
